@@ -29,7 +29,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 use std::fmt;
-use std::ops::{Add, Mul, Sub};
+use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
 use num::{Float, One, Zero, Num};
@@ -69,28 +69,11 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
     }
 
     fn transpose(&self) -> M44<T> {
-        let a1 = self[(0, 0)];
-        let a2 = self[(0, 1)];
-        let a3 = self[(0, 2)];
-        let a4 = self[(0, 3)];
-        let a5 = self[(1, 0)];
-        let a6 = self[(1, 1)];
-        let a7 = self[(1, 2)];
-        let a8 = self[(1, 3)];
-        let a9 = self[(2, 0)];
-        let a10 = self[(2, 1)];
-        let a11 = self[(2, 2)];
-        let a12 = self[(2, 3)];
-        let a13 = self[(3, 0)];
-        let a14 = self[(3, 1)];
-        let a15 = self[(3, 2)];
-        let a16 = self[(3, 3)];
-
         M44::new([
-            [a1, a5, a9, a13],
-            [a2, a6, a10, a14],
-            [a3, a7, a11, a15],
-            [a4, a8, a12, a16],
+            [self[(0, 0)], self[(1, 0)], self[(2, 0)], self[(3, 0)]],
+            [self[(0, 1)], self[(1, 1)], self[(2, 1)], self[(3, 1)]],
+            [self[(0, 2)], self[(1, 2)], self[(2, 2)], self[(3, 2)]],
+            [self[(0, 3)], self[(1, 3)], self[(2, 3)], self[(3, 3)]],
         ])
     }
 
@@ -153,28 +136,30 @@ impl<T: Float + std::iter::Sum> LinearAlgebra<T> for M44<T> {
         let a15 = self[(3, 2)];
         let a16 = self[(3, 3)];
 
-        a1 * a10 * a15 * a8 - a1 * a10 * a16 * a7 - a1 * a11 * a14 * a8
-            + a1 * a11 * a16 * a6
-            + a1 * a12 * a14 * a7
-            - a1 * a12 * a15 * a6
-            - a10 * a13 * a3 * a8
-            + a10 * a13 * a4 * a7
-            - a10 * a15 * a4 * a5
-            + a10 * a16 * a3 * a5
-            + a11 * a13 * a2 * a8
-            - a11 * a13 * a4 * a6
-            + a11 * a14 * a4 * a5
-            - a11 * a16 * a2 * a5
-            - a12 * a13 * a2 * a7
-            + a12 * a13 * a3 * a6
-            - a12 * a14 * a3 * a5
-            + a12 * a15 * a2 * a5
-            + a14 * a3 * a8 * a9
-            - a14 * a4 * a7 * a9
-            - a15 * a2 * a8 * a9
-            + a15 * a4 * a6 * a9
-            + a16 * a2 * a7 * a9
-            - a16 * a3 * a6 * a9
+        a1 * a10 * a15 * a8
+        - a1 * a10 * a16 * a7
+        - a1 * a11 * a14 * a8
+        + a1 * a11 * a16 * a6
+        + a1 * a12 * a14 * a7
+        - a1 * a12 * a15 * a6
+        - a10 * a13 * a3 * a8
+        + a10 * a13 * a4 * a7
+        - a10 * a15 * a4 * a5
+        + a10 * a16 * a3 * a5
+        + a11 * a13 * a2 * a8
+        - a11 * a13 * a4 * a6
+        + a11 * a14 * a4 * a5
+        - a11 * a16 * a2 * a5
+        - a12 * a13 * a2 * a7
+        + a12 * a13 * a3 * a6
+        - a12 * a14 * a3 * a5
+        + a12 * a15 * a2 * a5
+        + a14 * a3 * a8 * a9
+        - a14 * a4 * a7 * a9
+        - a15 * a2 * a8 * a9
+        + a15 * a4 * a6 * a9
+        + a16 * a2 * a7 * a9
+        - a16 * a3 * a6 * a9
     }
 
     /// Calculate the inverse
@@ -369,6 +354,17 @@ impl<T: Num + Copy> M44<T> {
         result
     }
 
+    /// Applies `f` of each element in the M44
+    pub fn for_each(&self, f: impl Fn(T) -> T) -> Self {
+        let mut result = Self::zeros();
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                result[(i, j)] = f(self[(i, j)]);
+            }
+        }
+        result
+    }
+
 }
 
 impl<T: Num + Copy> Add for M44<T> {
@@ -416,6 +412,13 @@ impl<T: Num + Copy> Add for M44<T> {
     }
 }
 
+// M44 += M44
+impl<T: Num + Copy> AddAssign for M44<T> {
+    fn add_assign(&mut self, other: Self) {
+        *self = *self + other
+    }
+}
+
 // M44 - M44
 impl<T: Num + Copy> Sub for M44<T> {
     type Output = Self;
@@ -463,6 +466,13 @@ impl<T: Num + Copy> Sub for M44<T> {
     }
 }
 
+// M44 -= M44
+impl<T: Num + Copy> SubAssign for M44<T> {
+    fn sub_assign(&mut self, other: Self) {
+        *self = *self - other
+    }
+}
+
 // M44 * constant
 impl<T: Num + Copy> Mul<T> for M44<T> {
     type Output = M44<T>;
@@ -1109,4 +1119,18 @@ mod test_matrix4x4 {
             &expected[..]
         );
     }
+
+    #[test]
+    fn for_each_test() {
+        let m = m44_new!( 0.0,  1.0,  2.0,  3.0;
+                          4.0,  5.0,  6.0,  7.0;
+                          8.0,  9.0, 10.0, 11.0;
+                         12.0, 13.0, 14.0, 15.0);
+        let result = m.for_each(|element| element + 1.0);
+        let expected = m44_new!( 1.0,  2.0,  3.0,  4.0;
+                                 5.0,  6.0,  7.0,  8.0;
+                                 9.0,  10.0, 11.0, 12.0;
+                                13.0, 14.0, 15.0, 16.0);
+        assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+    }
 }
