@@ -30,7 +30,7 @@
 //-------------------------------------------------------------------------
 use num::{Float, One, Zero, Num};
 use std::fmt;
-use std::ops::{Add, Mul, Sub};
+use std::ops::{Add, Mul, Sub, AddAssign, SubAssign};
 use std::ops::{Deref, DerefMut, Index, IndexMut};
 
 use crate::slices_methods::*;
@@ -410,6 +410,14 @@ impl<T: Num + Copy> Add for M55<T> {
     }
 }
 
+
+// M55 += M55
+impl<T: Num + Copy> AddAssign for M55<T> {
+    fn add_assign(&mut self, other: Self) {
+        *self = *self + other
+    }
+}
+
 // M55 - M55
 impl<T: Num + Copy> Sub for M55<T> {
     type Output = Self;
@@ -507,6 +515,13 @@ impl<T: Num + Copy> Sub for M55<T> {
     }
 }
 
+// M55 -= M55
+impl<T: Num + Copy> SubAssign for M55<T> {
+    fn sub_assign(&mut self, other: Self) {
+        *self = *self - other
+    }
+}
+
 // M55 * constant
 impl<T: Num + Copy> Mul<T> for M55<T> {
     type Output = M55<T>;
@@ -719,7 +734,6 @@ impl<T: Num + Copy> One for M55<T> {
     }
 }
 
-// NOTE(elsuizo:2020-04-26): poniendo ese Trait anda el norm2 funcional
 impl<T: Num + Copy> M55<T> {
     /// contruct identity matrix
     pub fn identity() -> M55<T> {
@@ -855,6 +869,17 @@ impl<T: Num + Copy> M55<T> {
         result
     }
 
+    /// Applies `f` of each element in the M44
+    pub fn for_each(&self, f: impl Fn(T) -> T) -> Self {
+        let mut result = Self::zeros();
+        for i in 0..self.rows() {
+            for j in 0..self.cols() {
+                result[(i, j)] = f(self[(i, j)]);
+            }
+        }
+        result
+    }
+
 }
 
 impl<T> Deref for M55<T> {
@@ -1292,4 +1317,23 @@ mod test_matrix5x5 {
         );
     }
 
+    #[test]
+    fn for_each_test() {
+        let m = m55_new!(10.0, 1.0, 7.0,  1.0,  5.0;
+                          2.0, 1.0, 8.0,  3.0,  2.0;
+                          5.0, 1.0, 1.0,  9.0, 10.0;
+                          6.0, 9.0, 9.0,  1.0,  3.0;
+                          1.0, 8.0, 8.0, 10.0,  5.0);
+
+        let result = m.for_each(|element| element + 1.0);
+
+        let expected = m55_new!(11.0, 2.0, 8.0,  2.0,  6.0;
+                                 3.0, 2.0, 9.0,  4.0,  3.0;
+                                 6.0, 2.0, 2.0,  10.0, 11.0;
+                                 7.0, 10.0, 10.0,  2.0,  4.0;
+                                 2.0, 9.0, 9.0, 11.0,  6.0);
+
+        assert!(compare_vecs(&result.as_vec(), &expected.as_vec(), EPS));
+    }
+
 }
