@@ -29,11 +29,11 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //-------------------------------------------------------------------------
 use std::fmt;
-use num::{Float, Zero, Num};
+use num::{Float, Zero, Num, Signed};
 use std::ops::{Deref, DerefMut};
 
-use std::ops::{Add, Mul, Sub, SubAssign, AddAssign};
-// use std::ops::{AddAssign, DivAssign, MulAssign, SubAssign};
+use std::ops::{Add, Sub, Div, Mul, SubAssign, AddAssign, Neg};
+use crate::slices_methods::{norm_l, norm_inf};
 
 use crate::errors::VectorErrors;
 use crate::matrix3x3::M33;
@@ -78,6 +78,48 @@ impl<T: Num + Copy> V3<T> {
                  u_x * self[1] - u_y * self[0]])
     }
 
+    pub fn x_axis() -> Self {
+        let one = T::one();
+        let zero = T::zero();
+        Self::new_from(one, zero, zero)
+    }
+
+    pub fn y_axis() -> Self {
+        let one = T::one();
+        let zero = T::zero();
+        Self::new_from(zero, one, zero)
+    }
+
+    pub fn z_axis() -> Self {
+        let one = T::one();
+        let zero = T::zero();
+        Self::new_from(zero, zero, one)
+    }
+
+
+}
+
+impl<T: Num + Copy + std::cmp::PartialOrd> V3<T> {
+    pub fn norm_inf(&self) -> T {
+        norm_inf(&**self)
+    }
+}
+
+impl<T: Num + Copy + Signed + std::iter::Sum> V3<T> {
+    pub fn norm_l(&self) -> T {
+        norm_l(&**self)
+    }
+}
+
+impl<T: Num + Copy + Signed> Neg for V3<T> {
+    type Output = Self;
+
+    fn neg(self) -> Self {
+        let a = self[0];
+        let b = self[1];
+        let c = self[2];
+        V3::new([-a, -b, -c])
+    }
 }
 
 impl<T: Float> V3<T> {
@@ -121,6 +163,18 @@ impl<T: Num + Copy> Mul<T> for V3<T> {
     }
 }
 
+// V3 / const
+impl<T: Num + Copy> Div<T> for V3<T> {
+    type Output = Self;
+
+    fn div(self, rhs: T) -> Self::Output {
+        let a0 = self[0] / rhs;
+        let a1 = self[1] / rhs;
+        let a2 = self[2] / rhs;
+        V3::new([a0, a1, a2])
+    }
+}
+
 // FIXME(elsuizo:2020-06-19): this is a hack
 impl Mul<V3<f32>> for f32 {
     type Output = V3<f32>;
@@ -430,4 +484,20 @@ mod vector3_test {
             &expected[..]
         );
     }
+
+    #[test]
+    fn norm_inf_test() {
+        let v = V3::new_from(1, -10, 73);
+        let result = v.norm_inf();
+        let expected = 73;
+        assert_eq!(result, expected);
+    }
+
+    #[test]
+    fn norm_l_test() {
+        let v = V3::new_from(1, -1, 1);
+        let result = v.norm_l();
+        let expected = 3;
+        assert_eq!(result, expected);
+    }
 }
